From 42aaa646e43b01d3d5e3cb127f5beb09c47d0d8a Mon Sep 17 00:00:00 2001
From: Or Goshen <orx.goshen@intel.com>
Date: Tue, 11 Feb 2014 16:11:51 +0200
Subject: [PATCH] Add a command to QMP to list the supported devices

---
 qapi-schema.json |   26 ++++++++++++++++++++++++++
 qdev-monitor.c   |   28 ++++++++++++++++++++++++++++
 qmp-commands.hx  |   40 ++++++++++++++++++++++++++++++++++++++++
 3 files changed, 94 insertions(+), 0 deletions(-)

diff --git a/qapi-schema.json b/qapi-schema.json
index 05ced9d..cae1200 100644
--- a/qapi-schema.json
+++ b/qapi-schema.json
@@ -526,6 +526,32 @@
 { 'command': 'query-commands', 'returns': ['CommandInfo'] }
 
 ##
+# @DeviceInfo:
+#
+# Information about a device
+#
+# @name: The command name
+# @bus: The bus it is connected to
+# @alias: Device alias
+# @desc: Description of the device
+#
+# Since: 1.6.0
+##
+{ 'type': 'DeviceInfo', 'data': {'name': 'str', 'bus': 'str', 'alias': 'str', 'desc': 'str'} }
+
+##
+# @query-devices:
+#
+# Return a list of supported devices
+#
+# Returns: A list of @DeviceInfo for all supported devices
+#
+# Since: 1.6.0
+##
+{ 'command': 'query-devices', 'returns': ['DeviceInfo'] }
+
+
+##
 # @EventInfo:
 #
 # Information about a QMP event
diff --git a/qdev-monitor.c b/qdev-monitor.c
index 1d3b68d..0a59fd9 100644
--- a/qdev-monitor.c
+++ b/qdev-monitor.c
@@ -644,6 +644,34 @@ void do_info_qtree(Monitor *mon, const QDict *qdict)
         qbus_print(mon, sysbus_get_default(), 0);
 }
 
+static void qdev_list_devinfo(ObjectClass *klass, void *data)
+{
+    DeviceInfoList *e, **pret = data;
+    DeviceInfo *info;
+    DeviceClass *dc = (DeviceClass *)object_class_dynamic_cast(klass, TYPE_DEVICE);
+    if (!dc) return;
+
+    info = g_malloc0(sizeof(*info));
+    info->name = g_strdup(object_class_get_name(klass));
+    info->bus = g_strdup(dc->bus_type ? dc->bus_type : "");
+    info->alias = g_strdup(qdev_class_has_alias(dc) ? qdev_class_get_alias(dc) : "");
+    info->desc = g_strdup(dc->desc ? dc->desc : "");
+
+    e = g_malloc0(sizeof(*e));
+    e->value = info;
+    e->next = *pret;
+    *pret = e;
+}
+
+DeviceInfoList * qmp_query_devices(Error **errp)
+{
+    DeviceInfoList *list = NULL;
+
+    object_class_foreach(qdev_list_devinfo, TYPE_DEVICE, false, &list);
+
+    return list;
+}
+
 void do_info_qdm(Monitor *mon, const QDict *qdict)
 {
     qdev_print_devinfos(true);
diff --git a/qmp-commands.hx b/qmp-commands.hx
index cce6b81..be4451d 100644
--- a/qmp-commands.hx
+++ b/qmp-commands.hx
@@ -1851,6 +1851,46 @@ EQMP
     },
 
 SQMP
+query-devices
+--------------
+
+List supported devices
+
+Each device is represented by a json-object, the returned value is a json-array
+of all devices.
+
+Each json-object contain:
+
+- "name": device's name (json-string)
+- "bus": bus the device connects to (json-string)
+- "alias": device's alias (json-string)
+- "desc": device's description (json-string)
+
+Example:
+
+-> { "execute": "query-devices" }
+<- {
+      "return":[
+         {
+            "name":"pci-bridge",
+            "bus":"PCI",
+            "alias":"",
+            "desc":"Standard PCI Bridge"
+         }
+      ]
+   }
+
+Note: This example has been shortened as the real response is too long.
+
+EQMP
+
+    {
+        .name       = "query-devices",
+        .args_type  = "",
+        .mhandler.cmd_new = qmp_marshal_input_query_devices,
+    },
+
+SQMP
 query-events
 --------------
 
-- 
1.7.9

